<?php
namespace Zeedhi\DevOps\Model\Configurationmanager\Repositories;

use Zeedhi\DevOps\Util\Utilities;

class DeployStep extends \Doctrine\ORM\EntityRepository {

    public function findDeployStep($customerCode, $filterQuery, $limit, $offset) {
        $query = <<<SQL
            SELECT *
              FROM (SELECT DPS.ID, DPS.PRODUCT_ID, DPS.PRODUCT_VERSION_ID, DPS.CDCLIENTE,
                           DPS.DEPLOY_DATE, DPS.DESCRIPTION, DPS.STATUS, DPS.IS_ACTIVE,
                           PVR.VERSION || '.' || PVR.REVISION AS VERSION, OMP.NMPRODUTO,
                           OMP.IDPRODUTOSAAS, OPO.IDPRODUTOSAAS AS IDPRODUTOORGSAAS, ORG.IDORGSAAS,
                           RTRIM(
                               XMLAGG(
                                   XMLELEMENT(E, SVR.NAME, ', ').EXTRACT('//text()') ORDER BY DSS.SERVER_ORDER
                               ).GetClobVal(), ', '
                           ) AS SERVER_LIST
                      FROM DEPLOY_STEP DPS
                      JOIN OM_PRODUTO OMP         ON OMP.NREXTERNAL_PRODUCT_ID = DPS.PRODUCT_ID
                      JOIN PRODUCT_VERSION PVR    ON PVR.ID = DPS.PRODUCT_VERSION_ID
                      JOIN DEPLOY_STEP_SERVER DSS ON DSS.DEPLOY_STEP_ID = DPS.ID
                      JOIN PRODUCT_SERVER PSR     ON PSR.ID = DSS.PRODUCT_SERVER_ID
                      JOIN SERVER SVR             ON SVR.ID = PSR.SERVER_ID
                      
                      JOIN CLIENTE CLI        ON CLI.CDCLIENTE = DPS.CDCLIENTE
                      JOIN ORGANIZACAO ORG    ON ORG.NRPARCNEGOCIO = CLI.NRPARCNEGOCIO
                      JOIN OM_PRODUTO_ORG OPO ON OPO.NRORG_ID = ORG.NRORG
                                             AND OPO.NRPRODUTO_ID = OMP.ID
                 
                     WHERE DPS.CDCLIENTE = :CDCLIENTE
                  GROUP BY DPS.ID, DPS.PRODUCT_ID, DPS.PRODUCT_VERSION_ID, DPS.CDCLIENTE,
                           DPS.DEPLOY_DATE, DPS.DESCRIPTION, DPS.STATUS, DPS.IS_ACTIVE,
                           PVR.VERSION || '.' || PVR.REVISION, OMP.NMPRODUTO,
                           OMP.IDPRODUTOSAAS, OPO.IDPRODUTOSAAS, ORG.IDORGSAAS)
             WHERE 1 = 1
                   $filterQuery
SQL;
        $params = array ( 'CDCLIENTE' => $customerCode );
        $query  = Utilities::getPaginatedQuery($query, $limit, $offset);
        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }
    
    public function findDeployStepById($id) {
        $query = <<<SQL
                SELECT DPS.ID, DPS.PRODUCT_ID, DPS.PRODUCT_VERSION_ID, DPS.CDCLIENTE,
                       DPS.DEPLOY_DATE, DPS.DESCRIPTION, DPS.STATUS, DPS.IS_ACTIVE,
                       PVR.VERSION || '.' || PVR.REVISION AS VERSION, OMP.NMPRODUTO, 
                       DPS.CREATED, OPC.NMOPERADOR AS CREATED_BY,
                       DPS.UPDATED, OPU.NMOPERADOR AS UPDATED_BY,
                       OMP.IDPRODUTOSAAS, OPO.IDPRODUTOSAAS AS IDPRODUTOORGSAAS, ORG.IDORGSAAS,
                       RTRIM(
                           XMLAGG(
                               XMLELEMENT(E, DSS.PRODUCT_SERVER_ID, ',').EXTRACT('//text()') ORDER BY DSS.SERVER_ORDER
                           ).GetClobVal(), ','
                       ) AS SERVER,
                       RTRIM(
                           XMLAGG(
                               XMLELEMENT(E, SVR.NAME, ', ').EXTRACT('//text()') ORDER BY DSS.SERVER_ORDER
                           ).GetClobVal(), ', '
                       ) AS SERVER_LIST,
                       CASE WHEN (SELECT COUNT(*)
                                    FROM DEPLOY_STEP_SERVER DST
                                    JOIN DEPLOY DPL ON DPL.ID = DST.DEPLOY_ID
                                   WHERE DST.DEPLOY_STEP_ID = DPS.ID
                                     AND DPL.STATUS = 'D') > 0
                            THEN 'N'
                            ELSE 'Y'
                        END AS CAN_BE_DELETED
                  FROM DEPLOY_STEP DPS
                  JOIN OM_PRODUTO OMP         ON OMP.NREXTERNAL_PRODUCT_ID = DPS.PRODUCT_ID
                  JOIN PRODUCT_VERSION PVR    ON PVR.ID = DPS.PRODUCT_VERSION_ID
                  JOIN DEPLOY_STEP_SERVER DSS ON DSS.DEPLOY_STEP_ID = DPS.ID
                  JOIN PRODUCT_SERVER PSR     ON PSR.ID = DSS.PRODUCT_SERVER_ID
                  JOIN SERVER SVR             ON SVR.ID = PSR.SERVER_ID
                  
             LEFT JOIN ZH_USER ZUC    ON ZUC.ID = DPS.CREATED_BY
             LEFT JOIN RESOURCE_T RTC ON RTC.ID = ZUC.EXTERNAL_RES_ID
             LEFT JOIN OPERADOR OPC   ON OPC.CDOPERADOR = RTC.EXTERNAL_CODE
                
             LEFT JOIN ZH_USER ZUU    ON ZUU.ID = DPS.UPDATED_BY
             LEFT JOIN RESOURCE_T RTU ON RTU.ID = ZUU.EXTERNAL_RES_ID
             LEFT JOIN OPERADOR OPU   ON OPU.CDOPERADOR = RTU.EXTERNAL_CODE
                  
                  JOIN CLIENTE CLI        ON CLI.CDCLIENTE = DPS.CDCLIENTE
                  JOIN ORGANIZACAO ORG    ON ORG.NRPARCNEGOCIO = CLI.NRPARCNEGOCIO
                  JOIN OM_PRODUTO_ORG OPO ON OPO.NRORG_ID = ORG.NRORG
                                         AND OPO.NRPRODUTO_ID = OMP.ID
                 WHERE DPS.ID = :ID
              GROUP BY DPS.ID, DPS.PRODUCT_ID, DPS.PRODUCT_VERSION_ID, DPS.CDCLIENTE,
                       DPS.DEPLOY_DATE, DPS.DESCRIPTION, DPS.STATUS, DPS.IS_ACTIVE, 
                       DPS.CREATED, OPC.NMOPERADOR, DPS.UPDATED, OPU.NMOPERADOR,
                       PVR.VERSION || '.' || PVR.REVISION, OMP.NMPRODUTO,
                       OMP.IDPRODUTOSAAS, OPO.IDPRODUTOSAAS, ORG.IDORGSAAS
SQL;
        $params = array ( 'ID' => $id );
        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }

    public function findMaxDeployStepServerDate($deployStepId) {
        $query = <<<SQL
                SELECT MAX(DP.DEPLOY_DATE) AS DEPLOY_DATE
                  FROM DEPLOY_STEP_SERVER DS
                  JOIN DEPLOY DP ON DP.ID = DS.DEPLOY_ID
                 WHERE DS.DEPLOY_STEP_ID = :DEPLOY_STEP_ID
                   AND DP.STATUS = 'D'
SQL;
        $params = array ( 'DEPLOY_STEP_ID' => $deployStepId );
        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }

    public function findMinDeployStepVersion($deployStepId, $productServerId) {
        $query = <<<SQL
                SELECT PV.ID, PV.PRODUCT_ID, PV.VERSION, PV.REVISION, PV.BUILD
                  FROM (SELECT PRODUCT_ID, MIN(VERSION_ORDER) AS VERSION
                          FROM (SELECT DP.PRODUCT_ID,
                                       TO_NUMBER(LPAD(PV.VERSION, 5, '0') || LPAD(PV.REVISION, 5, '0') || LPAD(PV.BUILD, 5, '0')) AS VERSION_ORDER
                                  FROM (SELECT D.ID
                                          FROM DEPLOY D
                                         WHERE D.IS_CUSTOMER_DEPLOY = 'Y'
                                    START WITH D.ID IN (SELECT DPL.ID
                                                          FROM DEPLOY_STEP_SERVER DSS
                                                          JOIN DEPLOY DPL ON DPL.DEPLOY_STEP_SRV_ID = DSS.ID
                                                         WHERE DSS.DEPLOY_STEP_ID = :DEPLOY_STEP_ID
                                                           AND DSS.PRODUCT_SERVER_ID = :PRODUCT_SERVER_ID)
                                       CONNECT BY PRIOR D.ID = D.PARENT) DT
                            INNER JOIN DEPLOY DP ON DP.ID = DT.ID
                            INNER JOIN PRODUCT_VERSION PV ON PV.ID = DP.PREVIOUS_PRODUCT_VERSION_ID)
                      GROUP BY PRODUCT_ID) VR          
                  JOIN PRODUCT_VERSION PV ON VR.PRODUCT_ID = PV.PRODUCT_ID
                                         AND TO_NUMBER(LPAD(PV.VERSION, 5, '0') || LPAD(PV.REVISION, 5, '0') || LPAD(PV.BUILD, 5, '0')) = VR.VERSION
SQL;
        $params = array ( 'DEPLOY_STEP_ID' => $deployStepId, 'PRODUCT_SERVER_ID' => $productServerId );
        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }

    public function findMaxDeployStepVersion($deployStepId, $productServerId) {
        $query = <<<SQL
                SELECT PV.ID, PV.PRODUCT_ID, PV.VERSION, PV.REVISION, PV.BUILD
                  FROM (SELECT PRODUCT_ID, MAX(VERSION_ORDER) AS VERSION
                          FROM (SELECT DP.PRODUCT_ID,
                                       TO_NUMBER(LPAD(PV.VERSION, 5, '0') || LPAD(PV.REVISION, 5, '0') || LPAD(PV.BUILD, 5, '0')) AS VERSION_ORDER
                                  FROM (SELECT D.ID
                                          FROM DEPLOY D
                                         WHERE D.IS_CUSTOMER_DEPLOY = 'Y'
                                    START WITH D.ID IN (SELECT DPL.ID
                                                          FROM DEPLOY_STEP_SERVER DSS
                                                          JOIN DEPLOY DPL ON DPL.DEPLOY_STEP_SRV_ID = DSS.ID
                                                         WHERE DSS.DEPLOY_STEP_ID = :DEPLOY_STEP_ID
                                                           AND DSS.PRODUCT_SERVER_ID = :PRODUCT_SERVER_ID)
                                       CONNECT BY PRIOR D.ID = D.PARENT) DT
                            INNER JOIN DEPLOY DP ON DP.ID = DT.ID
                            INNER JOIN PRODUCT_VERSION PV ON PV.ID = DP.PRODUCT_VERSION_ID)
                      GROUP BY PRODUCT_ID) VR          
                  JOIN PRODUCT_VERSION PV ON VR.PRODUCT_ID = PV.PRODUCT_ID
                                         AND TO_NUMBER(LPAD(PV.VERSION, 5, '0') || LPAD(PV.REVISION, 5, '0') || LPAD(PV.BUILD, 5, '0')) = VR.VERSION
SQL;
        $params = array ( 'DEPLOY_STEP_ID' => $deployStepId, 'PRODUCT_SERVER_ID' => $productServerId );
        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }

    public function findPendingDeployStepsByProductServer($productServerIds) {
        $query = <<<SQL
               SELECT DPS.ID, DSS.PRODUCT_SERVER_ID, DPS.STATUS, DPS.IS_ACTIVE
                 FROM DEPLOY_STEP_SERVER DSS
                 JOIN DEPLOY_STEP DPS ON DPS.ID = DSS.DEPLOY_STEP_ID
                WHERE DSS.PRODUCT_SERVER_ID IN ($productServerIds)
                  AND DPS.STATUS <> 'D'
                  AND DPS.IS_ACTIVE = 'Y'
SQL;
        return $this->getEntityManager()->getConnection()->fetchAll($query);
    }

    public function validateDeployPermition($productId) {
        $query = <<<SQL
               SELECT IDPERDEPLOY FROM OM_PRODUTO WHERE NREXTERNAL_PRODUCT_ID = $productId
SQL;
        return $this->getEntityManager()->getConnection()->fetchAll($query);
    }



}
