<?php
namespace Zeedhi\DevOps\Model\Projectmanagement\Repositories;

use Doctrine\DBAL\Connection;
use Zeedhi\DevOps\Model\Projectmanagement\Entities;

class TaskRelationship extends \Doctrine\ORM\EntityRepository {

    public function findRelatedProjectIds($task)
    {
        $query = <<<SQL
			SELECT MAIN_PROJECT,
			       RTRIM(XMLAGG(XMLELEMENT(e, P.RELATED_PROJECT, ',').EXTRACT('//text()') ORDER BY P.RELATED_PROJECT).GetClobVal(),',') AS RELATED_PROJECTS
              FROM (SELECT TA.PROJECT AS MAIN_PROJECT, TS.PROJECT AS RELATED_PROJECT
                      FROM TASK_RELATIONSHIP TR
                     INNER JOIN TASK TA ON TA.ID = TR.TASK
                     INNER JOIN TASK TS ON TS.ID = TR.TASK_RELATIONSHIP
                     WHERE TR.TYPE = :TYPE
                       AND TA.STATUS = :STATUS
                       AND TS.STATUS = :STATUS
                       AND TR.TASK IN (SELECT TL.TASK
                                         FROM TASK_RELATIONSHIP TL
                                        WHERE (TL.TASK = :TASK OR TL.TASK_RELATIONSHIP = :TASK)
                                          AND TL.TYPE = :TYPE)) P
             GROUP BY MAIN_PROJECT
SQL;
        $params = array(
            'TASK'   => $task,
            'STATUS' => Entities\Task::ACTIVE,
            'TYPE'   => Entities\TaskRelationship::RELATED_TASK_TYPE
        );
        $result = $this->getEntityManager()->getConnection()->fetchAll($query, $params);
        return !empty($result) ? $result[0] : $result;
    }

    public function findRelatedProjectIdsWithResourceTaskFilter($task, $currentTaskId){
        $query = <<<SQL
			SELECT MAIN_PROJECT,
			       RTRIM(XMLAGG(XMLELEMENT(e, P.RELATED_PROJECT, ',').EXTRACT('//text()') ORDER BY P.RELATED_PROJECT).GetClobVal(),',') AS RELATED_PROJECTS
              FROM (SELECT DECODE(ALTA.RESOURCE_T, NULL, NULL, TA.PROJECT) AS MAIN_PROJECT,
                           DECODE(ALTS.RESOURCE_T, NULL, NULL, TS.PROJECT) AS RELATED_PROJECT
                      FROM TASK_RELATIONSHIP TR
                     INNER JOIN ALLOCATION AL ON AL.TASK = :CURRENT_TASK_ID
                     INNER JOIN TASK TA ON TA.ID = TR.TASK
                     INNER JOIN TASK TS ON TS.ID = TR.TASK_RELATIONSHIP
                      LEFT JOIN PROJECT TAP ON TA.PROJECT = TAP.ID
                      LEFT JOIN PROJECT TAS ON TS.PROJECT = TAS.ID
                      LEFT JOIN PROJECT_TYPE TAPT ON TAP.PROJECT_TYPE = TAPT.ID
                      LEFT JOIN PROJECT_TYPE TAPS ON TAS.PROJECT_TYPE = TAPS.ID
                      LEFT JOIN TASK_RELATIONSHIP TRTA ON TRTA.TASK = TA.ID AND (TRTA.TYPE = 'M' OR TAPT.USEMERGEREQUEST = 'N')
                      LEFT JOIN TASK_RELATIONSHIP TRTS ON TRTS.TASK = TS.ID AND (TRTS.TYPE = 'M' OR TAPS.USEMERGEREQUEST = 'N')
                      LEFT JOIN ALLOCATION ALTA ON ALTA.TASK = TRTA.TASK_RELATIONSHIP AND ALTA.RESOURCE_T = AL.RESOURCE_T
                      LEFT JOIN ALLOCATION ALTS ON ALTS.TASK = TRTS.TASK_RELATIONSHIP AND ALTS.RESOURCE_T = AL.RESOURCE_T
                     WHERE TR.TYPE = :TYPE
                       AND TA.STATUS = :STATUS
                       AND TS.STATUS = :STATUS
                       AND TR.TASK IN (SELECT TL.TASK
                                         FROM TASK_RELATIONSHIP TL
                                        WHERE (TL.TASK = :TASK OR TL.TASK_RELATIONSHIP = :TASK)
                                          AND TL.TYPE = :TYPE)) P
             GROUP BY MAIN_PROJECT
SQL;
        $params = array(
            'TASK' => $task,
            'CURRENT_TASK_ID' => $currentTaskId,
            'STATUS' => Entities\Task::ACTIVE,
            'TYPE' => Entities\TaskRelationship::RELATED_TASK_TYPE
        );
        $result = $this->getEntityManager()->getConnection()->fetchAll($query, $params);
        return !empty($result) ? $result[0] : $result;
    }

    public function findRelatedTaskIds($task, $status = Entities\Task::ACTIVE){
        $query = <<<SQL
			SELECT MAIN_TASK,
			       RTRIM(XMLAGG(XMLELEMENT(e, P.RELATED_TASK, ',').EXTRACT('//text()') ORDER BY P.RELATED_TASK).GetClobVal(),',') AS RELATED_TASKS
              FROM (SELECT TA.ID AS MAIN_TASK, TS.ID AS RELATED_TASK
                      FROM TASK_RELATIONSHIP TR
                     INNER JOIN TASK TA ON TA.ID = TR.TASK
                     INNER JOIN TASK TS ON TS.ID = TR.TASK_RELATIONSHIP
                     WHERE TR.TYPE = :TYPE
                       AND TA.STATUS = :STATUS
                       AND TS.STATUS = :STATUS
                       AND TR.TASK IN (SELECT TL.TASK
                                         FROM TASK_RELATIONSHIP TL
                                        WHERE (TL.TASK = :TASK OR TL.TASK_RELATIONSHIP = :TASK)
                                          AND TL.TYPE = :TYPE)) P
             GROUP BY MAIN_TASK
SQL;
        $params = array(
            'TASK'   => $task,
            'STATUS' => $status,
            'TYPE'   => Entities\TaskRelationship::RELATED_TASK_TYPE
        );
        $result = $this->getEntityManager()->getConnection()->fetchAll($query, $params);
        return !empty($result) ? $result[0] : $result;
    }
    
    
    public function findNrparcnegocioByCdoperator($cdoperador){
        $query = <<<SQL
			SELECT NRPARCNEGOCIO 
			  FROM OPERADOR
			 WHERE CDOPERADOR = :CDOPERADOR
SQL;
        $params = array(
            'CDOPERADOR'   => $cdoperador
        );
        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
        
    }

    public function findRelatedTaskByTaskIds($taskIds){
        $query = <<<SQL
			SELECT (
            SELECT RTRIM(XMLAGG(XMLELEMENT(e, P.RELATED_TASK, ',').EXTRACT('//text()') ORDER BY P.RELATED_TASK).GetClobVal(),',')||','||MAIN_TASK AS RELATED_TASKS
              FROM (SELECT TA.ID AS MAIN_TASK, TS.ID AS RELATED_TASK
                      FROM TASK_RELATIONSHIP TR
                     INNER JOIN TASK TA ON TA.ID = TR.TASK
                     INNER JOIN TASK TS ON TS.ID = TR.TASK_RELATIONSHIP
                     WHERE TR.TYPE = :TYPE
                       AND TA.STATUS = :STATUS
                       AND TS.STATUS = :STATUS
                       AND TR.TASK IN (SELECT TL.TASK
                                         FROM TASK_RELATIONSHIP TL
                                        WHERE (TL.TASK = TAS.ID OR TL.TASK_RELATIONSHIP = TAS.ID)
                                          AND TL.TYPE = :TYPE)) P
             GROUP BY MAIN_TASK)AS RELATED_TASKS,TAS.ID AS TASK
             FROM TASK TAS
             WHERE TAS.ID IN (:TASKS)
SQL;
        $params = array(
            'TASKS'   => $taskIds,
            'STATUS' => Entities\Task::ACTIVE,
            'TYPE'   => Entities\TaskRelationship::RELATED_TASK_TYPE
        );

        $paramTypes = array(
            'TASKS' => Connection::PARAM_STR_ARRAY
        );
        $result = $this->getEntityManager()->getConnection()->fetchAll($query, $params, $paramTypes);
        return $result;
    }
    
    public function findLastEndDateTask($taskId, $idProject, $userId){
        $query = <<<SQL
            SELECT ID FROM (
                SELECT T.ID 
                FROM TASK T
                INNER JOIN ALLOCATION A ON A.TASK = T.ID
                WHERE T.PROJECT = :PROJECT 
                    AND T.STATUS = :STATUS
                    AND T.ID NOT IN (:TASK)
                    AND T.IS_PARENT = :NOT_IS_PARENT
                    AND A.RESOURCE_T = 5704
                ORDER BY ESTIMATED_END_DATE DESC)
            WHERE ROWNUM <=1
SQL;

        $params = array(
            'TASK'   => $taskId,
            'STATUS' => Entities\Task::ACTIVE,
            'NOT_IS_PARENT' => Entities\Task::NOT_IS_PARENT,
            'PROJECT'   => $idProject,
            'USER'  => $userId
        );
        $result = $this->getEntityManager()->getConnection()->fetchAll($query, $params);
        return !empty($result) ? $result[0] : $result;
    }
    
    public function findRepositorysOfEachProject($tasksRelatedId){
        $query = <<<SQL
            SELECT T.ID
            FROM REPOSITORY R 
             JOIN TASK T ON T.PROJECT = R.PROJECT
             JOIN DIRECTORY D ON D.REPOSITORY = R.ID
            WHERE T.ID IN (:TASKS)
            AND R.STATUS <> 'D'
            GROUP BY T.ID
SQL;

        $params = array(
            'TASKS'   => $tasksRelatedId,
            'STATUS' => Entities\Task::ACTIVE
        );
        
        $paramTypes = array(
            'TASKS' => Connection::PARAM_STR_ARRAY
        );
        
        $result = $this->getEntityManager()->getConnection()->fetchAll($query, $params, $paramTypes);
        return $result;
    }

    public function getAllPrecursors($taskId){
        $query = <<<SQL
            SELECT TA.ID
            FROM TASK T
            JOIN TASK_RELATIONSHIP TR ON T.ID IN (TR.TASK_RELATIONSHIP,TR.TASK)
            JOIN TASK TA ON TA.ID IN (TR.TASK_RELATIONSHIP,TR.TASK)
            WHERE T.ID = :TASK
            AND TA.ID <> :TASK
            AND T.STATUS = 'A'
            GROUP BY TA.ID
SQL;

        $params = array(
            'TASK'   => $taskId
        );

        $result = $this->getEntityManager()->getConnection()->fetchAll($query, $params);
        return $result;
    }

    public function checkAllRelatedTasks($realTaskIds){
        $query = <<<SQL
        SELECT T.ID 
        FROM TASK T
        WHERE T.ID IN (
            SELECT TRE.TASK
            FROM TASK_RELATIONSHIP TRE
            WHERE TRE.TASK_RELATIONSHIP IN ( SELECT DISTINCT TR.TASK_RELATIONSHIP 
                                            FROM TASK_RELATIONSHIP TR
                                            WHERE (TR.TASK IN (:ids) OR TASK_RELATIONSHIP IN (:ids))
                                            AND TYPE in (:FIRST_TYPE))
            UNION
            SELECT DISTINCT TR.TASK_RELATIONSHIP 
            FROM TASK_RELATIONSHIP TR
            WHERE (TR.TASK IN (:ids) OR TASK_RELATIONSHIP IN (:ids))
            AND TYPE in (:FIRST_TYPE)
        )
        AND T.PROJECT = (SELECT MAX(PROJECT) FROM TASK WHERE ID IN (:ids))
SQL;

        $params = array(
            'ids' => $realTaskIds,
            'FIRST_TYPE' => Entities\TaskRelationship::FIRST_TYPE
        );
        $paramTypes = array(
            'ids' => Connection::PARAM_STR_ARRAY
        );
        $result = $this->getEntityManager()->getConnection()->fetchAll($query, $params, $paramTypes);
        return $result;
    }

    public function findComplementaryRelatedTasksByTaskId($taskId)
    {
        $query = 
             "SELECT TASK || ',' || RTRIM(XMLAGG(XMLELEMENT(e, TASK_RELATIONSHIP, ',').EXTRACT('//text()') ORDER BY TASK_RELATIONSHIP).GetClobVal(),',') AS TASKS
               FROM (SELECT TR.TASK, TR.TASK_RELATIONSHIP
                       FROM TASK_RELATIONSHIP TR
                       JOIN TASK TA ON TA.ID = TR.TASK
                       JOIN TASK TS ON TS.ID = TR.TASK_RELATIONSHIP
                      WHERE TR.TASK IN (SELECT TR.TASK
                                          FROM TASK_RELATIONSHIP TR
                                         WHERE TR.TASK = :TASK_ID OR TR.TASK_RELATIONSHIP = :TASK_ID
                                           AND TYPE = 'A')
                        AND TA.STATUS = :STATUS
                        AND TS.STATUS = :STATUS)
           GROUP BY TASK";
        $params = array(
            'TASK_ID'   => $taskId,
            'STATUS' => Entities\Task::ACTIVE,
            'TYPE'   => Entities\TaskRelationship::RELATED_TASK_TYPE
        ); 
        $result = $this->getEntityManager()->getConnection()->fetchAll($query, $params);
        return !empty($result) ? $result[0] : $result;
    }    

}