/************************************************************************************
****    Procedure para calculo do demonstrativo de resultado operacional
*************************************************************************************

Criado em  : 03/2018
       Por : Jefferson Gouvea

Descrio:

          - Com base nos dados parametrizados para a unidade realiza apurao do
            resultado operacional da mesma, grava em tabelas especficas a fim de
            possibilitar a gerao de relatrios gerenciais a partir de dados
            previamente calculados.
*/

CREATE OR REPLACE PROCEDURE APURAC_INVESTROP(
   P_CDFILIAL       FILIAL.CDFILIAL%TYPE,
   P_FILIALPAR      FILIAL.CDFILIAL%TYPE,
   P_DTINICIAL      IN DATE,
   P_DTFINAL        IN DATE,
   P_NRROPFILIAL    IN VARCHAR)
IS
   V_NRSEQINVEROP       NUMBER;
   V_IDAPUINVFILROP     CHAR;
   V_IDAPURAINVCTPAGROP CHAR;
   V_CDFILCENTPME       VARCHAR2(04);
BEGIN
   BEGIN
      SELECT IDAPUINVFILROP, IDAPURAINVCTPAGROP
        INTO V_IDAPUINVFILROP, V_IDAPURAINVCTPAGROP
        FROM PARFILIALROP P
       WHERE P.CDFILIAL = P_FILIALPAR;
   EXCEPTION
      WHEN NO_DATA_FOUND THEN BEGIN
         V_IDAPUINVFILROP := NULL;
         V_IDAPURAINVCTPAGROP := 'N';
      END;
   END;

   BEGIN
     SELECT CASE WHEN IDFILCENTPME = 'S'
                 THEN CDFILIAL
                 ELSE CDFILCENTPME
            END
       INTO V_CDFILCENTPME
       FROM FILIAL
      WHERE CDFILIAL = P_CDFILIAL;
   EXCEPTION
     WHEN NO_DATA_FOUND THEN
       V_CDFILCENTPME := NULL;
   END;

   IF V_IDAPUINVFILROP = 'S' THEN
      FOR C_LANCINVEFILI IN (SELECT CDINVEST, DTINVEFILI, VRINVEFILI, CDFILIAL
                               FROM LANCINVEFILI
                              WHERE CDFILIAL = P_CDFILIAL                         
                                AND TRUNC(DTINVEFILI) = P_DTINICIAL) LOOP

         NOVO_CODIGO('GESINVESTIMENTOROP', V_NRSEQINVEROP);

         INSERT INTO INVESTIMENTOROP
                    (NRFILIALROP, NRINVESTIMENTOROP, IDTPLANCROP, DTINVESTIMENTO, DTLANCAMENTO,
                     CDFILCENTPME, CDINVEST, QTINVEST, VRINVEST, CDFILIALNF,
                     NRLANCTONF, NRSEQVENPAG, NRPARCELAPAG)
              VALUES
                    (P_NRROPFILIAL, V_NRSEQINVEROP, 'INVESTIMENTO', C_LANCINVEFILI.DTINVEFILI, SYSDATE,
                     V_CDFILCENTPME, C_LANCINVEFILI.CDINVEST, 1, C_LANCINVEFILI.VRINVEFILI, NULL,
                     NULL, NULL, NULL);
      END LOOP;
   END IF;   
    
   IF V_IDAPURAINVCTPAGROP = 'S' THEN
      FOR C_CTPAGINV IN (SELECT CP.IDSTATPAG, CP.CDFILIBPAG, CP.CDFILIAL,CP.NRLANCTONF,CP.NRSEQVENPAG, CP.NRPARCELAPAG,
                                TC.CDTPCTPAGAR,(CP.DTATUAVENPAG)DTBAIXAPAG,NVL(VRRATCCCTPAG,CP.VRATUAPAG) AS VRCONTARATEADA
                           FROM CONTAPAGAR CP INNER JOIN NOTAFISCAL NF ON NF.CDFILIAL = CP.CDFILIAL
                                                                      AND NF.NRLANCTONF = CP.NRLANCTONF
                                              INNER JOIN BOLETPAGAR BO ON CP.CDFILIBPAG = BO.CDFILIAL
                                                                      AND CP.NRLANBOLPAG = BO.NRLANBOLPAG
                                              LEFT  JOIN BAIXAPAGA BP ON BO.CDFILIBAIPAG = BP.CDFILIAL
                                                                     AND BO.NRBAIXAPAG = BP.NRBAIXAPAG
                                              INNER JOIN TIPOCTPAGROP TC ON TC.CDTPCTPAGAR = CP.CDTPCTPAGAR
                                              LEFT  JOIN RATCTPAGAR R ON CP.CDFILIAL = R.CDFILIAL
                                                                     AND CP.NRLANCTONF = R.NRLANCTONF
                                                                     AND CP.NRSEQVENPAG = R.NRSEQVENPAG
                                                                     AND CP.NRPARCELAPAG = R.NRPARCELAPAG
                          WHERE TRUNC(CP.DTATUAVENPAG) = P_DTINICIAL
                            AND TC.CDFILIAL = P_FILIALPAR
                            AND TC.IDTPCTPAGROP = 'INV'
                            AND ((CDFIRATCTPAG = P_CDFILIAL) OR (CDFIRATCTPAG IS NULL) AND (NF.CDFILIRECE = P_CDFILIAL))) LOOP

         NOVO_CODIGO('GESINVESTIMENTOROP', V_NRSEQINVEROP);

         INSERT INTO INVESTIMENTOROP
                    (NRFILIALROP, NRINVESTIMENTOROP, IDTPLANCROP, DTINVESTIMENTO, DTLANCAMENTO,
                     CDFILCENTPME, CDINVEST, QTINVEST, VRINVEST, CDFILIALNF,
                     NRLANCTONF, NRSEQVENPAG, NRPARCELAPAG)
              VALUES
                    (P_NRROPFILIAL, V_NRSEQINVEROP, 'INVESTIMENTO', C_CTPAGINV.DTBAIXAPAG, SYSDATE,
                     V_CDFILCENTPME, C_CTPAGINV.CDTPCTPAGAR, 1, C_CTPAGINV.VRCONTARATEADA, C_CTPAGINV.CDFILIAL,
                     C_CTPAGINV.NRLANCTONF, C_CTPAGINV.NRSEQVENPAG, C_CTPAGINV.NRPARCELAPAG);
      END LOOP;
   END IF;
END;
/

-- VERSAO --
BEGIN ATUVEROBJ('APURAC_INVESTROP','6.29.xxx'); END;
/
